package org.globus.transfer.reliable.service.test.connection;

import java.util.Date;

import junit.framework.TestCase;
import org.globus.rft.generated.RFTOptionsType;
import org.ietf.jgss.GSSCredential;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ThirdPartyConnectionImpl;
import org.globus.transfer.reliable.service.exception.RftTransientException;

public class ThirdPartyConnectionImplTest extends TestCase {
    
    private ThirdPartyConnectionImpl conn;
    private RFTOptionsType rftOptions1;
    private RFTOptionsType rftOptions2;
    private ConnectionParameter params1;
    private ConnectionParameter params2;
    private String sourceSubject;
    private String destSubject;
    private GSSCredential credential = null;

    public ThirdPartyConnectionImplTest(String name) {
        
        super(name);
    }

    protected void setUp() throws Exception {
        
        super.setUp();
        this.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        // create different RFTOptionsType objects
        rftOptions1 = new RFTOptionsType();
        rftOptions2 = new RFTOptionsType();
        rftOptions1.setDcau(new Boolean(true));
        rftOptions2.setDcau(new Boolean(false));

        this.sourceSubject = TestConfig.sourceSubject;
        if (this.sourceSubject == null || this.sourceSubject.trim().equals("")) {
            this.sourceSubject = TestConfig.subject;
        }
        this.destSubject = TestConfig.destinationSubject;
        if (this.destSubject == null || this.destSubject.trim().equals("")) {
            this.destSubject = TestConfig.subject;
        }
        
        // create different connection parameter objects
        params1 = new ConnectionParameter(
            TestConfig.sourceHost, TestConfig.sourcePort,
            TestConfig.sourcePath, TestConfig.destinationHost,
            TestConfig.destPort, TestConfig.destPath, this.sourceSubject,
            rftOptions1);
        params2 = new ConnectionParameter(
            TestConfig.destinationHost, TestConfig.destPort,
            TestConfig.destPath, TestConfig.sourceHost,
            TestConfig.sourcePort, TestConfig.sourcePath, this.sourceSubject,
            rftOptions2);
        
        this.conn = new ThirdPartyConnectionImpl(params1,credential,
            this.sourceSubject, this.destSubject);
    }

    protected void tearDown() {
      
        this.conn.closeConnection();
    }

    public void testGetType() {
        
        assertTrue(this.conn.getType() ==
                ConnectionManager.THIRDPARTY_CONNECTION);
    }

    public void testGetNumberFDs() {
        
        assertTrue(this.conn.getNumberFDs() == 2);
    }

    public void testCheckForReuse() {
        
        assertTrue(this.conn.checkForReuse(this.params1));
        assertFalse(this.conn.checkForReuse(this.params2));
    }

    public void testCheckConnectionIntegrity() {
        
        assertTrue(this.conn.checkConnectionIntegrity());
    }
    
    public void testSetRestartMarker() {
        
        // checking invalid restart marker string
        try {
            this.conn.setRestartMarker("1");
            fail("restart marker string is malformed and should fail");
        } catch(RftTransientException e) {
        } catch(Exception e) {
            fail("expecting RftTransientException and not "
                + e.getClass().getName());
        }
        
        // checking valid restart marker string
        try {
            this.conn.setRestartMarker("1-42");
            this.conn.setRestartMarker("1-42,1-345");
        } catch(Exception e) {
            fail("no exception expected for valid restart marker string");
        }
    }

    public void testUpdateForReuse() {
        
        // not much we can test here
        try {
            this.conn.updateForReuse(this.params2);
        } catch(Exception e) {
            fail("testUpdateForReuse() failed: " + e);
        }
    }

    public void testGetSourcePath() {
        
        System.out.println("sourcePath1: " + params1.getSourcePath());
        System.out.println("sourcePath2: " + this.conn.getSourcePath());
        assertTrue(this.conn.getSourcePath().equals(params1.getSourcePath()));
        this.conn.updateForReuse(params2);
        assertFalse(this.conn.getSourcePath().equals(params1.getSourcePath()));
        assertTrue(this.conn.getSourcePath().equals(params2.getSourcePath()));
    }

    public void testGetDestinationPath() {
        
        assertTrue(this.conn.getDestinationPath().equals(params1.getDestinationPath()));
        this.conn.updateForReuse(params2);
        assertFalse(this.conn.getDestinationPath().equals(params1.getDestinationPath()));
        assertTrue(this.conn.getDestinationPath().equals(params2.getDestinationPath()));
    }

    public void testUpdateLastUsed()
    {
        assertTrue(this.conn.getLastUsed() == 0);
        long now1 = new Date().getTime();
        this.conn.updateLastUsed();
        long lastUsed = this.conn.getLastUsed();
        long now2 = new Date().getTime();
        assertTrue((now1 <= lastUsed) && (lastUsed <= now2));
    }

    public void testNormalExtendedTransfer() {
        
        try {
            TestUtil.removeFileIfExists(params1.getDestinationHost(),
                params1.getDestinationPort(), params1.getDestinationPath(),
                this.sourceSubject, this.credential);
            this.conn.normalExtendedTransfer(null);
            if (!TestUtil.removeFileIfExists(params1.getDestinationHost(),
                params1.getDestinationPort(), params1.getDestinationPath(),
                this.sourceSubject, this.credential)) {
                fail("file didn't exist on destination after transfer");
            }
        } catch(Exception e) {
            fail("testNormalExtendedTransfer() failed: " + e);
        }
    }

    public void testNormalNonExtendedTransfer() {
        
        try {
            TestUtil.removeFileIfExists(params1.getDestinationHost(),
                params1.getDestinationPort(), params1.getDestinationPath(),
                this.sourceSubject, this.credential);
            this.conn.normalNonExtendedTransfer(null);
            if (!TestUtil.removeFileIfExists(params1.getDestinationHost(),
                params1.getDestinationPort(), params1.getDestinationPath(),
                this.sourceSubject, this.credential)) {
                fail("file didn't exist after transfer");
            }
        } catch(Exception e) {
            fail("testNormalExtendedTransfer() failed: " + e);
        }
    }
    
/*
    public void testDoMlstOnSource() {
        
    }
    
    public void testSetPermissionsOnDestination() {
        
    }
*/
}
